""" init """
from wenxin_api.tasks.text_to_image import TextToImage
from wenxin_api.tasks.cloze import Cloze
from wenxin_api.tasks.composition import Composition
from wenxin_api.tasks.couplet import Couplet
from wenxin_api.tasks.free_qa import FreeQA
from wenxin_api.tasks.novel import Novel
from wenxin_api.tasks.official_documents import OfficialDocuments
from wenxin_api.tasks.summarization import Summarization
from wenxin_api.tasks.text_generation import TextGeneration


""" api init """
from wenxin_api.api.base_api import APIBaseObject
from wenxin_api.api.base_api import (CreatableAPIObject, 
                                     StopableAPIObject,
                                     DeletableAPIObject, 
                                     ListableAPIObject, 
                                     RetrievalableAPIObject
                                    )
from wenxin_api.api.task_api import Task
from wenxin_api.api.model_api import Model
from wenxin_api.api.dataset_api import Dataset
from wenxin_api.api.train_api import Train


# !/usr/bin/env python3
""" task api """
import time
import json
from wenxin_api.api.base_api import CreatableAPIObject, ListableAPIObject, StopableAPIObject, RetrievalableAPIObject
from wenxin_api.api.model_api import Model
from wenxin_api.base_object import BaseObject
from wenxin_api.error import APIError, IllegalRequestArgumentError, MissingRequestArgumentError
from wenxin_api.const import INFERENCE_TASK
from wenxin_api.const import (CMD_DO_DEPLOY, 
                              CMD_DO_INFERENCE, 
                              CMD_QUERY_MODEL, 
                              CMD_QUERY_TASK,
                              CMD_STOP_TASK,)
from wenxin_api.const import (MODEL_STATE_ERROR, 
                              MODEL_STATE_SUCCESS, 
                              MODEL_STATE_ON_SERVICE,)
from wenxin_api.const import (TASK_STATE_INFER_IN_QUEUE,
                              TASK_STATE_INFER_RUNNING,
                              TASK_STATE_INFER_END,)
from wenxin_api.const import (TASK_STATE_DEPLOY_SUBMIT, 
                              TASK_STATE_DEPLOY_IN_QUEUE, 
                              TASK_STATE_DEPLOY_RUNNING,
                              TASK_STATE_DEPLOY_SUCCESS,
                              TASK_STATE_DEPLOY_FAILED,)
from wenxin_api.const import ERNIE_100B_MODEL
from wenxin_api.variable import REQUEST_SLEEP_TIME
from wenxin_api import log
logger = log.get_logger()

class Task(CreatableAPIObject, ListableAPIObject, StopableAPIObject, RetrievalableAPIObject):
    """ task class """
    OBJECT_NAME = "tasks"

    @staticmethod
    def _resolve_result(resp):
        rst = {}
        rst["result"] = resp["output"]["result"]
        return rst

    @classmethod
    def create(cls, text=None, model=None, **params):
        if text is None:
            raise IllegalRequestArgumentError("text shouldn't be none")

        if model is None:
            # provide by default
            model = Model.retrieve(model_id=ERNIE_100B_MODEL)
        else:
            model.update()
        # deploy
        if model.status != MODEL_STATE_ON_SERVICE:
            request_id = CMD_DO_DEPLOY
            params["type"] = "task"
            deploy_task = super().create(model_id=model.id, request_id=request_id, **params)

            not_ready = True
            while not_ready:
                deploy_task.update()
                not_ready = deploy_task.status != TASK_STATE_DEPLOY_SUCCESS
                if deploy_task.status == TASK_STATE_DEPLOY_FAILED:
                    raise APIError("deploy task:{} failed".format(deploy_task.id))
                logger.info("model is preparing now!, task_id:{}, status:{}".format(
                    deploy_task.id, 
                    deploy_task.status)
                )
                time.sleep(REQUEST_SLEEP_TIME)
        
        # inference
        logger.info("model {}: starts writing".format(model.id))
        request_id = CMD_DO_INFERENCE
        # todo:add base_model and is_prompt
        # api_type 10 for inference
        resp = cls.default_request(text=text, 
                                   model_id=model.id, 
                                   api_type=10, 
                                   request_id=request_id, 
                                   **params)
        return cls._resolve_result(resp)

    @classmethod
    def list(cls, *args, **params):
        """ list """
        request_id = CMD_QUERY_TASK
        params["type"] = "task"
        resps = super().list(request_id=request_id, **params)
        filtered_resps = [resp for resp in resps \
                            if resp.status >= 300 and \
                               resp.status < 400]
        return filtered_resps

    @classmethod
    def retrieve(cls, *args, **params):
        """ retrieve """
        request_id = CMD_QUERY_TASK
        params["type"] = "task"
        if "task_id" not in params:
            raise MissingRequestArgumentError("task_id is not provided")
        return super().retrieve(request_id=request_id, **params)

    def stop(self, *args, **params) -> BaseObject:
        request_id = CMD_STOP_TASK
        params["type"] = "task"
        params["task_type"] = INFERENCE_TASK
        params["task_id"] = self.id
        if self.id is None:
            raise IllegalRequestArgumentError("illegal task_id")
        super().stop(request_id=request_id, **params)
        self.update()

    def update(self, *args, **params):
        """ update """
        request_id = CMD_QUERY_TASK
        params["type"] = "task"
        task = super().retrieve(task_id=self.id, request_id=request_id, **params)
        self.refresh_from(task)

    def __str__(self):
        return "Task {}:{}".format(
                        id(self),
                        json.dumps({"id": self.id, 
                                    "status": self.status,
                                    "type": self.type
                                   }, ensure_ascii=False)
        )

    def __repr__(self):
        return self.__str__()





#   Copyright (c) 2019 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Arguments for configuration."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import six
import argparse
import logging
import os


def str2bool(v):
    """
    str2bool
    """
    # because argparse does not support to parse "true, False" as python
    # boolean directly
    return v.lower() in ("true", "t", "1")


class ArgumentGroup(object):
    """ArgumentGroup"""

    def __init__(self, parser, title, des):
        self._group = parser.add_argument_group(title=title, description=des)

    def add_arg(self, name, type, default, help, positional_arg=False, **kwargs):
        """add_arg"""
        prefix = "" if positional_arg else "--"
        type = str2bool if type == bool else type
        self._group.add_argument(
            prefix + name,
            default=default,
            type=type,
            help=help + ' Default: %(default)s.',
            **kwargs)


def print_arguments(args):
    """print_arguments"""
    logging.info("-----------  Configuration Arguments -----------")
    for arg, value in sorted(six.iteritems(vars(args))):
        logging.info("%s: %s" % (arg, value))
    logging.info("------------------------------------------------")


def build_common_arguments():
    """build_common_arguments"""
    parser = argparse.ArgumentParser(__doc__)
    model_g = ArgumentGroup(parser, "model", "model configuration and paths.")
    model_g.add_arg("mode", str, "train", "train,inference,eval")
    model_g.add_arg("param_path", str, None, "path to parameter file describing the model to be trained")
    model_g.add_arg("paddle_version", str, "1.5.2", "paddle_fluid version code")
    model_g.add_arg("pre_train_type", str, None, "type of pretrain mode:ernie_base, "
                                                 "ernie_large, ernie_tiny, ernie_distillation, None")
    model_g.add_arg("task_type", str, "custom", "task type:classify, matching, sequence_label, generate, custom")
    model_g.add_arg("net_type", str, "custom", "net type: CNN,BOW,TextCNN,CRF, LSTM, SimNet-BOW ...")
    model_g.add_arg("run_script", str, "run_trainer.py", "run_trainer.py for preprocess to run")

    args = parser.parse_args()
    return args